

################################################################################


class DetailsMagnetostaticField:
  
  
  exists = False
  
  bemSpace = None
  bemToNgIdx = None
  bemLhs = None
  bemRhsOp = None
  
  
#------------------------------------------------------------------------------#


  def __init__(self, mesh, NeumannSpace, DirichletSpace, comp2, solvetol):
    from mpi4py import MPI
    import bempp.api
    import sys
    
    if type(self).exists:
      print("Error from DetailsMagnetostaticField.__init__()")
      print("An object of this class already exists! --> Terminate")
      sys.exit()

    type(self).exists = True

    # TODO: what values should we choose here?
    bempp.api.global_parameters.quadrature.double_singular = 11
    bempp.api.global_parameters.quadrature.near.double_order = 6 
    bempp.api.global_parameters.quadrature.medium.double_order = 6 
    bempp.api.global_parameters.quadrature.far.double_order = 6 
    bempp.api.global_parameters.quadrature.far.single_order = 6 
    bempp.api.global_parameters.assembly.boundary_operator_assembly_type = 'hmat'
    bempp.api.global_parameters.hmat.max_rank = 1000
    bempp.api.global_parameters.hmat.eps = 1.0e-10

    type(self)._bemStaticMagnetostaticInitialize(NeumannSpace)

  
#------------------------------------------------------------------------------#


  @classmethod
  def _bemStaticMagnetostaticInitialize(cls, ngNeumannSpace):
    ## ng-space --> bem-space
    import sys
    from mpi4py import MPI
    import bempp.api
    import ngbem
    import numpy as np

    try:
      ngScalarNeumannSpace = ngNeumannSpace.components[0]
    except:
      ngScalarNeumannSpace = ngNeumannSpace

    cls.bemSpace, op_NgToBem = ngbem.H1_trace(ngScalarNeumannSpace)

    op_NgToBem.eliminate_zeros()
    if (op_NgToBem.shape[0] != op_NgToBem.data.size):
      print("from DetailsMagnetostaticField._bemMagnetostaticInitialize:")
      print("in ngbem: H1 --> H1_trace went wrong! ---> Terminate!")
      sys.exit()

    cls.bemToNgIdx = np.full(op_NgToBem.shape[0] \
      , fill_value = ngScalarNeumannSpace.ndof, dtype=int)
    cls.bemToNgIdx[op_NgToBem.row] = op_NgToBem.col
    
    ## double layer K operator
    bem_K = bempp.api.operators.boundary.laplace.double_layer(cls.bemSpace \
      , cls.bemSpace, cls.bemSpace)
    
    ## identity I operator
    bem_I = bempp.api.operators.boundary.sparse.identity(cls.bemSpace \
      , cls.bemSpace, cls.bemSpace)
    
    
    ## building RHS
    cls.bemRhsOp = bem_K - 0.5*bem_I
    
    # building LHS
    cls.bemLhs = bem_I
  
  
#------------------------------------------------------------------------------#


  @classmethod
  def bemMagnetostaticComputeBCs(cls, ng_comp1, ngNeumannSpace, solvetol):
    from ngsolve import GridFunction
    from mpi4py import MPI
    import bempp.api
    
    if ngNeumannSpace.type == "h1ho":
      dim = 1
    elif ngNeumannSpace.type == "VectorH1":
      dim = 3
    else:
      print("Error from DetailsMagnetostaticField.bemMagnetostaticComputeBCs()")
      print("Expected >h1ho< or >VectorH1< as ngNeumannSpace.type, but got" \
        , NeumannSpace.type)
      sys.exit()

    ## ng_comp1 --> bem_comp1
    bem_comp1 = []
    bem_rhs = []
    bem_g = []
    for d in range(dim):
      bem_comp1.append(bempp.api.GridFunction(cls.bemSpace\
        , coefficients=ng_comp1.vec.FV().NumPy()[cls.bemToNgIdx]))
    
      bem_rhs.append(cls.bemRhsOp * bem_comp1[d]) #comp. time bottleneck
    
      # solve using gmres
      bem_g.append((bempp.api.linalg.iterative_solvers.gmres( \
        cls.bemLhs, bem_rhs[d], tol=solvetol, maxiter=4000 ))[0])
      
    ng_g = GridFunction(ngNeumannSpace)
    if ng_g.dim == 1:
      ng_g.vec.FV().NumPy()[ cls.bemToNgIdx ] = bem_g[0].coefficients
    else:
      for d in range(dim):
        ng_g.components[d].vec.FV().NumPy()[ cls.bemToNgIdx ] = bem_g[d].coefficients
    
    return ng_g


#------------------------------------------------------------------------------#


################################################################################
