

################################################################################


def StructuredMesh_Cuboid(Xmin, Xmax, h):
  from commics._setup.geometries.standardGeometries._structuredMeshing.addStructuredMesh_MicroCuboid \
    import AddStructuredMesh_MicroCuboid
  from commics._setup.geometries.standardGeometries._structuredMeshing.addStructuredMesh_BoundaryFaceCuboid \
    import AddStructuredMesh_BoundaryFaceCuboid

  from netgen.meshing import MeshPoint, Mesh, Pnt, FaceDescriptor
  import numpy as np
  
  Ncells = [0, 0, 0]
  Nnodes = [0, 0, 0]
  for d in range(3):
    Ncells[d] = round((Xmax[d]-Xmin[d])/h[d])
    Nnodes[d] = Ncells[d]+1
  
  ngmesh = Mesh()
  ngmesh.dim = 3
  
  nodes = np.empty(Nnodes, dtype=object)
  for nx in range(Nnodes[0]):
    for ny in range(Nnodes[1]):
      for nz in range(Nnodes[2]):
        nodes[nx, ny, nz] = ngmesh.Add(MeshPoint( \
          Pnt(Xmin[0]+nx*h[0], Xmin[1]+ny*h[1], Xmin[2]+nz*h[2])))
  
  
  for cx in range(Ncells[0]):
    for cy in range(Ncells[1]):
      for cz in range(Ncells[2]):
        AddStructuredMesh_MicroCuboid(ngmesh, nodes[cx:cx+2, cy:cy+2, cz:cz+2])
  
  fd = ngmesh.Add (FaceDescriptor(bc=1,domin=1,domout=0,surfnr=1))
  
  AddStructuredMesh_BoundaryFaceCuboid(ngmesh, fd, nodes[0, :, :], "xmin")
  AddStructuredMesh_BoundaryFaceCuboid(ngmesh, fd, nodes[-1, :, :], "xmax")
  AddStructuredMesh_BoundaryFaceCuboid(ngmesh, fd, nodes[:, 0, :], "ymin")
  AddStructuredMesh_BoundaryFaceCuboid(ngmesh, fd, nodes[:, -1, :], "ymax")
  AddStructuredMesh_BoundaryFaceCuboid(ngmesh, fd, nodes[:, :, 0], "zmin")
  AddStructuredMesh_BoundaryFaceCuboid(ngmesh, fd, nodes[:, :, -1], "zmax")
  
  # for unknown reasons, it seems this number (0) has to be the bc parameter in 
  # FaceDescriptor, but -1.     ...
  ngmesh.SetBCName(0, "bc_dirichlet")

  return ngmesh


################################################################################
