from . import _standardGeometry


################################################################################


class Cuboid(_standardGeometry._StandardGeometry):
  '''
  cuboid
  two possibilities:
  A) input1 = (1,2,3)
     input2 = (4,5,6)
     input3 = None
     --> Cuboid with diagonal (1,2,3)--(4,5,6)
  A) input1 = 3
     input2 = 4
     input3 = 5
     --> Cuboid with dimension 3x4x5 centered around (0,0,0)
  '''



#------------------------------------------------------------------------------#


  def __init__(self, input1, input2, input3=None):
    self._input1 = input1
    self._input2 = input2
    self._input3 = input3


#------------------------------------------------------------------------------#


  def _GetNgSolveGeometry(self):
    from netgen.csg import OrthoBrick, Pnt
  
    p1, p2 = self.GetXminXmax()
  
    P1 = Pnt(p1[0], p1[1], p1[2])
    P2 = Pnt(p2[0], p2[1], p2[2])
    return OrthoBrick(P1, P2).bc("bc_dirichlet")


#------------------------------------------------------------------------------#


  def GetXminXmax(self):
    if isinstance(self._input1, tuple) and isinstance(self._input2, tuple):
      return self._input1, self._input2
    else:
      try:
        return (-self._input1/2, -self._input2/2, -self._input3/2) \
          , (self._input1/2, self._input2/2, self._input3/2)
      except:
        print("From Cuboid -- inputs must be 2 (float, float, float) tuples \
          or 3 floats dimX, dimY, dimZ as inputs")
        raise TypeError


#------------------------------------------------------------------------------#


################################################################################
