import matplotlib.pyplot as plt
import numpy as np
import sys
# little script for plotting multiple files with same structure.
# use: e.g., python3 compareFiles.py result1/averages.dat result2/averages.dat result3/averages.dat

colors = ['b', 'g', 'r', 'c', 'm', 'y', 'pink', 'olivedrab', 'brown', 'silver', 'orange']
commentMarker = "#"
headerSeparation = ","

if ( len(sys.argv) <= 1 ):
  print("error - no input filename.")
  sys.exit()

referenceFile = sys.argv[1]

B = np.loadtxt(referenceFile, comments="#")

# set labels
n = B.shape[1]-1
labels = np.empty(n+1, dtype=object)
f = open(referenceFile, "r")
header = f.readline()
f.close()
if ( header[0] != commentMarker ):
  for j in range(n+1):
    labels[j] = "data_"+str(j)
else:
  for j in range(n+1):
    idx1 = header.find(headerSeparation)
    tmp = header[:idx1]
    idx2 = tmp.rfind(" ")
    labels[j] = tmp[idx2+1:]
    header = header[idx1+1:]
xlabel = labels[0]
ylabels = labels[1:]
# end set labels

print(ylabels)

fig, axarr = plt.subplots(n, sharex=True)
if ( n==1):
  axarr.plot(B[:,0], B[:,1], 'k', label=sys.argv[1])
  axarr.set_title(ylabels[0])
else:
  for k in range(n):
    axarr[k].plot(B[:,0], B[:,k+1], 'k', label=sys.argv[1])
    axarr[k].set_title(ylabels[k])

for j in range(2, len(sys.argv)):
  resultFile = sys.argv[j]
  A = np.loadtxt(resultFile, comments=commentMarker)
  if(n==1):
    axarr.plot(A[:,0], A[:,1], color=colors[ j % len(colors) ], label=sys.argv[j] )
  else:
    for k in range(n):
      axarr[k].plot(A[:,0], A[:,k+1], color=colors[ j % len(colors) ], label=sys.argv[j] )

if(n==1):
    axarr.legend(loc="best", fontsize="small")
else:
  for k in range(n):
    axarr[k].legend(loc="best", fontsize="small")

plt.show()
