### Class for Corrector=Midpoint-form(Heun-like), 
### consider Derivative v is already computed ###
from . import _forms


################################################################################


class _CorrectorMidpoint( \
  _forms.lf_magnetization.LinearForms_Magnetization, \
  _forms.lf_derivative.LinearForms_Derivative):


#------------------------------------------------------------------------------#


  def _StepUpdate(self):
    import numpy as np

    self._ComputeEta()

    m_old_gf = np.array(self._Mag.m.vec.FV())
    self._Mag.m.vec.FV().NumPy()[:] = \
      2.0 * self._Mag_eta.m.vec.FV().NumPy() - m_old_gf


#------------------------------------------------------------------------------#


  def _SetUpAuxiliaryFunctions(self):
    from ngsolve import GridFunction

    super()._SetUpAuxiliaryFunctions()

    self._SetUpEta()

    self._heff_m_kv_gf = GridFunction(self._X)

    self._eta_guess = None
    self._eta_preconditioner = None


#------------------------------------------------------------------------------#


  def _SetUpEta(self):
    from commics._gridFunctions import Magnetization

    self._Mag_eta = Magnetization(self._geometry, self._V, self._X \
      , self._Mag, self.parameters.maxwellCoupling, self._solvetol)


#------------------------------------------------------------------------------#

  
  def _InitializeOerstedField(self):
    super()._InitializeOerstedField()

    from commics._gridFunctions import oerstedField, _zeroField
    if self.parameters.spintronicsCoupling != "none" \
      and (self.parameters.maxwellCoupling == "oerstedField" \
      or self.parameters.maxwellCoupling == "magnetostaticField"):
      self._hc_mp = oerstedField.OerstedField(self._geometry.GetMesh() \
        , self._j_mp_cf_oersted, self._solvetol, self._flag_je_stationary)
    else:
      self._hc_mp = _zeroField.ZeroField()


#------------------------------------------------------------------------------#


  def _UpdateOerstedFieldFlags(self):
    super()._UpdateOerstedFieldFlags()
    try:
      self._hc_mp.UpdateFlags(self._flag_je_stationary)
    except(AttributeError):
      pass


#------------------------------------------------------------------------------#


  def _UpdateMaxwellCoupling(self):

    super()._UpdateMaxwellCoupling()

    self._Mag_eta.UpdateMaxwellCoupling(self.parameters.maxwellCoupling \
      , self._solvetol)

    self._hc_mp = None
    from commics._gridFunctions import oerstedField, _zeroField
    if self.parameters.spintronicsCoupling != "none" \
      and (self.parameters.maxwellCoupling == "oerstedField" \
      or self.parameters.maxwellCoupling == "magnetostaticField"):
      self._hc_mp = oerstedField.OerstedField(self._geometry.GetMesh() \
        , self._j_mp_cf_oersted, self._solvetol, self._flag_je_stationary)
    else:
      self._hc_mp = _zeroField.ZeroField()


#------------------------------------------------------------------------------#


  def _ComputeMaxwell(self):
    super()._ComputeMaxwell()

    self._maxwellTimer.Start()
    self._hc_mp.Compute(self._solvetol)
    self._maxwellTimer.Stop()


#------------------------------------------------------------------------------#


  def _SetUpSystems(self):
    import numpy as np
    import scipy.sparse as sp
    from commics._tools import ScipySparseFromNgMatrix
    from ngsolve import IntegrationRule, BilinearForm, SymbolicBFI \
      , LinearForm, SymbolicLFI, TaskManager

    super()._SetUpSystems()

    massLumping = IntegrationRule( \
      points = [(0,0,0), (1,0,0), (0,1,0), (0,0,1)] \
      , weights = [1/24, 1/24, 1/24, 1/24] )

    ### Mh Mh^-1 begin
    mh_cf = self.__MassTerm()

    self._mh = BilinearForm(self._X)
    self._mh += SymbolicBFI(mh_cf.Compile(realcompile=True, wait=True) \
      , intrule=massLumping)

    with TaskManager():
      self._mh.Assemble()
    self._Mh = ScipySparseFromNgMatrix(self._mh)

    self._Mh_diag = self._Mh.diagonal()
    self._MhInv_diag = 1.0 / self._Mh_diag
    self._MhInv = sp.csr_matrix((self._MhInv_diag \
      , (np.arange(self._MhInv_diag.size), np.arange(self._MhInv_diag.size))))

    self._Mh.eliminate_zeros()
    ### Mh Mh^-1 end

    #auxiliary
    aux_heff_m_parts = -self.__ExchangeTerm()
    aux_heff_m_parts += self._LinearTerms_pi()
    aux_heff_m_parts += self._NonLinearTerms_Pi()
    aux_heff_m_parts += self._ExplicitTermsMidpoint()

    aux_heff_v_parts = -self._DerivExchangeTermMidpoint()
    aux_heff_v_parts += self._DerivLinearTerms_piMidpoint()
    aux_heff_v_parts += self._DerivNonLinearTerms_PiMidpoint()

    aux_heff_parts = aux_heff_m_parts + aux_heff_v_parts

    self._aux_f_parts = LinearForm(self._X)
    self._aux_f_parts += SymbolicLFI(aux_heff_parts.Compile( \
      realcompile=True, wait=True))


    #blf
    self._update_LHS_stat = self._Get_Update_LhsMass_Matrix()

    update_lhs_instat_cf = self._Update_Lhs_Skew_heff_m_kv_Term()
    update_lhs_instat_cf += self._UpdateSkewTerm()

    self._update_a = BilinearForm(self._X)
    self._update_a += SymbolicBFI(update_lhs_instat_cf.Compile( \
      realcompile=True, wait=True), intrule=massLumping)

    with TaskManager():
      self._update_a.Assemble()
    self._update_LHS_instat = ScipySparseFromNgMatrix(self._update_a)
    

    #lf
    update_rhs_cf = self._Update_RhsMassTerm()

    self._update_f = LinearForm(self._X)
    self._update_f += SymbolicLFI(update_rhs_cf.Compile( \
      realcompile=True, wait=True), intrule=massLumping)


#------------------------------------------------------------------------------#

  
  def _ComputeEta(self):
    self._ComputeUpdateSystem()
    self._SolveUpdateSystem()


#------------------------------------------------------------------------------#


  def _ComputeUpdateSystem(self):
    from ngsolve import TaskManager

    with TaskManager():
      self._Mag.v.ComputeMaxwell(self._solvetol)

      self._Compute_Update_LHS()
      self._Compute_Update_RHS()


#------------------------------------------------------------------------------#


  def _Compute_Update_LHS(self):
    self._AssembleUpdatePartsLHS()
    self._heff_m_kv_gf.vec.FV().NumPy()[:] \
      = self._MhInv_diag * self._aux_f_parts.vec.FV().NumPy()
  
    self._AssembleUpdateLHS()

    self._update_LHS = self._update_LHS_stat + self._update_LHS_instat


#------------------------------------------------------------------------------#


  def _AssembleUpdatePartsLHS(self):
    self._aux_f_parts.Assemble()


#------------------------------------------------------------------------------#


  def _AssembleUpdateLHS(self):
    self._update_a.Assemble()
    self._update_LHS_instat.data = self._update_a.mat.COO()[2].NumPy()


#------------------------------------------------------------------------------#


  def _Compute_Update_RHS(self):
    self._AssembleUpdateRHS()
    
    self._update_RHS = self._update_f.vec.FV().NumPy()


#------------------------------------------------------------------------------#


  def _AssembleUpdateRHS(self):
    self._update_f.Assemble()


#------------------------------------------------------------------------------#


  def _SolveUpdateSystem(self):
    import scipy.sparse.linalg

    self._Mag_eta.m.vec.FV().NumPy()[:], succ = scipy.sparse.linalg.gmres( \
      self._update_LHS, self._update_RHS \
      , x0=self._eta_guess, tol=self._solvetol, maxiter=4000 \
      , M=self._eta_preconditioner, callback=self._callback)

    self._UpdateEtaGuess()
    

#------------------------------------------------------------------------------#


  def _UpdateEtaGuess(self): pass


#------------------------------------------------------------------------------#


  def _ExplicitTermsMidpoint(self):
    return self._Applied_mp() + self._OerstedFieldTerm_mp()


#------------------------------------------------------------------------------#


  def _OerstedFieldTerm_mp(self):
    return self._Oersted(self._hc_mp)


#------------------------------------------------------------------------------#
  

  def _Get_Update_LhsMass_Matrix(self):
    return (2.0 / self._k) * self._Mh


#------------------------------------------------------------------------------#


  def _Update_RhsMassTerm(self):
    return (2.0 / self._k) * self._RhsMass()


#------------------------------------------------------------------------------#


  def _Update_Lhs_Skew_heff_m_kv_Term(self):
    return -self._Skew_heff_m_kv()
    

#------------------------------------------------------------------------------#


  def _Skew_heff_m_kv(self):
    return self._Skew(self._heff_m_kv_gf)


#------------------------------------------------------------------------------#


  def _UpdateSkewTerm(self):
    return -(2.0 * self.parameters.alpha / self._k) \
      * self._UpdateSkew()


#------------------------------------------------------------------------------#


  def _UpdateSkew(self):
    return self._Skew(self._Mag.m)


#------------------------------------------------------------------------------#


  def __ExchangeTerm(self):
    return self._C_ex * self._Exchange(self._Mag.Dm)


#------------------------------------------------------------------------------#

  
  def __MassTerm(self):
    return self._Mass()


#------------------------------------------------------------------------------#


################################################################################
