from . import _interface

################################################################################


class CommunicationInterface(_interface._Interface):


#------------------------------------------------------------------------------#


  def __str__(self):
    string = "\n\n -------------------- "
    string += "Integrator: " + self.__class__.__name__ 
    string += " --------------------" + "\n\n"
    string += "numthreads = " + str(self._numthreads) + "\n\n"
    
    for key, value in self.__dict__.items():
      if not(key.startswith("_") or key in self._nonTrivialMembers):
        string += key + " = " + str(value) + "\n"
    
    for key in self._nonTrivialMembers:
      string += self.__getattribute__(key).__str__() + "\n"

    string += "\n\n -------------------"
    string += "--------------------"
    string += "--------------------"
    string += "--------------------" + "\n\n"
    return string


#------------------------------------------------------------------------------#


  def _GetMesh(self):
    return self._geometry.GetMesh()
  

#------------------------------------------------------------------------------#


  def PrintTime(self):
    print("t = %.3e" % self.Time(), "s")
  
  
#------------------------------------------------------------------------------#


  def Time(self):
    return self._t / self.parameters.Ms / self.parameters.gamma0
  
  
#------------------------------------------------------------------------------#

  
  def GetTime(self):
    return self.Time()


#------------------------------------------------------------------------------#


  def _EvaluateExternalFieldAtPoint(self):
    import types
    from ngsolve import x, y, z, CoefficientFunction
    if self.parameters.H_ext == None:
      return [0.0, 0.0, 0.0]

    # TODO also full .vtk or chosen point (mip) should be possible
    ext = [0, 0, 0]
    for j in range(3):
      if isinstance(self.parameters.H_ext[j], types.FunctionType):
        time = self._time_cf / self.parameters.Ms / self.parameters.gamma0
        cf = CoefficientFunction(self.parameters.H_ext[j](time, x, y, z))
        pnt = self._externalFieldRecordingPoint
        mip = self._geometry.GetMesh()(pnt[0], pnt[1], pnt[2])
        ext[j] = cf(mip)
      else:
        ext[j] = self.parameters.H_ext[j]
    
    return ext


#------------------------------------------------------------------------------#


  def _EvaluateCurrent(self):
    import types
    import sys
    from ngsolve import CoefficientFunction
    if self.parameters.spintronicsCoupling == "none":
      return [0.0, 0.0, 0.0, 0.0]

    zero = CoefficientFunction(0.0)
    cur = [0, 0, 0, 0]
    if self.parameters.spintronicsCoupling == "slonczewski":
      if isinstance(self.parameters.Je, types.FunctionType):
        time = self._time_cf / self.parameters.Ms / self.parameters.gamma0
        cf = CoefficientFunction(self.parameters.Je(time, zero, zero, zero))
        pnt = self._geometry.GetMesh().ngmesh.Points()[1].p
        mip = self._geometry.GetMesh()(pnt[0], pnt[1], pnt[2])
        cur[0] = cf(mip)
      else:
        cur[0] = self.parameters.Je
    elif self.parameters.spintronicsCoupling == "zhangLi":
      for j in range(3):
        if isinstance(self.parameters.Je[j], types.FunctionType):
          time = self._time_cf / self.parameters.Ms / self.parameters.gamma0
          cf = CoefficientFunction(self.parameters.Je[j](time, zero, zero, zero))
          pnt = self._geometry.GetMesh().ngmesh.Points()[1].p
          mip = self._geometry.GetMesh()(pnt[0], pnt[1], pnt[2])
          cur[j+1] = cf(mip)
        else:
          cur[j+1] = self.parameters.Je[j]
    else:
      print("From _EvaluateCurrent(): spintronicsCoupling invalid")
      print("--> Terminate")
      sys.exit()
    
    return cur


#------------------------------------------------------------------------------#


################################################################################
