### collection of basic linear forms ###


################################################################################


class LinearForms_Common:
  

#------------------------------------------------------------------------------#


  def _Exchange(self, Dgf):
    from . import _linearForms
    return _linearForms.Exchange(self._Dphi, Dgf)


#------------------------------------------------------------------------------#


  def _UniaxialAnisotropy(self, gf):
    from . import _linearForms
    if(self.parameters.K == 0.0):
      return self._ZeroCF()

    return _linearForms.UniaxialAnisotropy(self._phi, self._easy_cf \
      , gf)


#------------------------------------------------------------------------------#


  def _DzyaloshinskiiMoriya(self, gf, Dgf):
    import sys
    if(self.parameters.dmCoupling == "none"):
      return self._ZeroCF()

    if(self.parameters.dmCoupling == "bulk"):
      return self._BulkDMI(gf, Dgf)
    elif(self.parameters.dmCoupling == "interfacial"):
      return self._InterfacialDMI(gf, Dgf)
    else:
      print("This option is not implemented.")
      sys.exit()


#------------------------------------------------------------------------------#


  def _Stray(self, gf_obj):
    from . import _linearForms

    if( self.parameters.maxwellCoupling != "magnetostaticField" \
      and self.parameters.maxwellCoupling != "strayField"):
      return self._ZeroCF()

    return _linearForms.Stray(self._phi, gf_obj.hs.cf)


#------------------------------------------------------------------------------#


  def _Oersted(self, hc_obj):
    from . import _linearForms

    if( self.parameters.maxwellCoupling != "magnetostaticField" \
      and self.parameters.maxwellCoupling != "oerstedField"):
      return self._ZeroCF()

    return _linearForms.Oersted(self._phi, hc_obj.cf)


#------------------------------------------------------------------------------#


  def _BulkDMI(self, gf, Dgf):
    from . import _linearForms
    return _linearForms.BulkDMI(self._phi, gf, self._Dphi, Dgf)


#------------------------------------------------------------------------------#


  def _InterfacialDMI(self, gf, Dgf):
    from . import _linearForms
    return _linearForms.InterfacialAxisDMI(self._phi, gf, self._Dphi, Dgf \
      , self.parameters.dmi_axis)


#------------------------------------------------------------------------------#

  
  def _Spintronics(self, gf, Dgf):
    import sys
    if(self.parameters.spintronicsCoupling == "none"):
      return self._ZeroCF()

    if(self.parameters.spintronicsCoupling == "zhangLi"):
      return self._ZhangLi(gf, Dgf)
    elif(self.parameters.spintronicsCoupling == "slonczewski"):
      return self._Slonczewski(gf)
    else: 
      print("This option is not implemented.")
      sys.exit()


#------------------------------------------------------------------------------#


  def _ZhangLi(self, gf, Dgf):
    from . import _linearForms
    return _linearForms.ZhangLiFirst(self._phi, self._j_cf, Dgf, gf) \
      + self.parameters.xi \
      * _linearForms.ZhangLiSecond(self._phi, self._j_cf, Dgf)


#------------------------------------------------------------------------------#


  def _Slonczewski(self, gf):
    from . import _linearForms
    return _linearForms.SlonczewskiFirst(self._phi, self._j_cf \
      , self.parameters.P, self._p_cf, gf, gf)


#------------------------------------------------------------------------------#


  def _Applied(self):
    from . import _linearForms
    if self.parameters.H_ext == None:
      return self._ZeroCF()
    else:
      return _linearForms.Applied(self._phi, self._hext_cf)


#------------------------------------------------------------------------------#


  def _Applied_impl(self):
    from . import _linearForms
    if self.parameters.H_ext == None:
      return self._ZeroCF()
    else:
      return _linearForms.Applied(self._phi, self._hext_impl_cf)


#------------------------------------------------------------------------------#


  def _Applied_mp(self):
    from . import _linearForms
    if self.parameters.H_ext == None:
      return self._ZeroCF()
    else:
      return _linearForms.Applied(self._phi, self._hext_mp_cf)


#------------------------------------------------------------------------------#


  def _LfMass(self, gf):
    from . import _linearForms
    return _linearForms.Mass(self._phi, gf)


#------------------------------------------------------------------------------#


################################################################################
