### collection of Derivative linear forms ###


################################################################################


class LinearForms_Derivative:
  

#------------------------------------------------------------------------------#

  
  def _DerivLinearTerms_pi(self):
    return self._theta * self._k * \
      ( self._DerivUniaxialAnisotropyTerm() \
      + self._DerivStrayFieldTerm() \
      + self._DerivDzyaloshinskiiMoriyaTerm() )


#------------------------------------------------------------------------------#

  
  def _DerivLinearTerms_pi_Implicit_RHS(self):
    return self._theta * self._k * \
      ( self._DerivUniaxialAnisotropyTerm() \
      + self._DerivStrayFieldTerm() \
      + self._DerivDzyaloshinskiiMoriyaTerm_Implicit_RHS() )


#------------------------------------------------------------------------------#


  def _DerivLinearTerms_piMidpoint(self):
    return 0.5 * self._k * \
      ( self._DerivUniaxialAnisotropyTerm() \
      + self._DerivStrayFieldTerm() \
      + self._DerivDzyaloshinskiiMoriyaTerm() )


#------------------------------------------------------------------------------#


  def _DerivExchangeTermMidpoint(self):
    return self._C_ex * 0.5 * self._k * self._DerivExchange()


#------------------------------------------------------------------------------#


  def _DerivExchange(self):
    return self._Exchange(self._Mag.v.Dgf)


#------------------------------------------------------------------------------#


  def _DerivUniaxialAnisotropyTerm(self):
    return -self._C_ani * self._DerivUniaxialAnisotropy()


#------------------------------------------------------------------------------#


  def _DerivStrayFieldTerm(self):
    return self._DerivStrayField()


#------------------------------------------------------------------------------#


  def _DerivDzyaloshinskiiMoriyaTerm(self):
    return -0.5 * self._C_dm * self._DerivDzyaloshinskiiMoriya()


#------------------------------------------------------------------------------#


  def _DerivDzyaloshinskiiMoriyaTerm_Implicit_RHS(self):
    if self.parameters.dmImplicitTreatment:
      return self._ZeroCF()
    else:
      return -0.5 * self._C_dm * self._DerivDzyaloshinskiiMoriya()


#------------------------------------------------------------------------------#


  def _DerivNonLinearTerms_Pi(self):
    return self._theta * self._k * self._DerivSpintronicsTerm()


#------------------------------------------------------------------------------#


  def _DerivNonLinearTerms_PiMidpoint(self):
    return 0.5 * self._k * self._DerivSpintronicsTerm()


#------------------------------------------------------------------------------#


  def _DerivSpintronicsTerm(self):
    return self._C_st \
      * self._DerivSpintronics(self._Mag.m, self._Mag.v.gf \
      , self._Mag.Dm, self._Mag.v.Dgf)


#------------------------------------------------------------------------------#


  def _DerivUniaxialAnisotropy(self):
    return self._UniaxialAnisotropy(self._Mag.v.gf)


#------------------------------------------------------------------------------#


  def _DerivStrayField(self):
    return self._Stray(self._Mag.v)


#------------------------------------------------------------------------------#


  def _DerivDzyaloshinskiiMoriya(self):
    return self._DzyaloshinskiiMoriya(self._Mag.v.gf, self._Mag.v.Dgf)


#------------------------------------------------------------------------------#


  def _DerivSpintronics(self, gf1, gf2, Dgf1, Dgf2):
    import sys
    if(self.parameters.spintronicsCoupling == "none"):
      return self._ZeroCF()

    if(self.parameters.spintronicsCoupling == "zhangLi"):
      return self._DerivZhangLi(gf1, gf2, Dgf1, Dgf2)
    elif(self.parameters.spintronicsCoupling == "slonczewski"):
      return self._DerivSlonczewski(gf1, gf2)
    else: 
      print("This option is not implemented.")
      sys.exit()
    

#------------------------------------------------------------------------------#


  def _DerivZhangLi(self, gf1, gf2, Dgf1, Dgf2):
    from . import _linearForms
    return _linearForms.ZhangLiFirst(self._phi, self._j_cf, Dgf1, gf2) \
      + _linearForms.ZhangLiFirst(self._phi, self._j_cf, Dgf2, gf1) \
      + _linearForms.ZhangLiFirst(self._phi, self._jprime_cf, Dgf1, gf1) \
      + self.parameters.xi \
      * _linearForms.ZhangLiSecond(self._phi, self._j_cf, Dgf2) \
      + self.parameters.xi \
      * _linearForms.ZhangLiSecond(self._phi, self._jprime_cf, Dgf1)


#------------------------------------------------------------------------------#


  def _DerivSlonczewski(self, gf1, gf2):
    from . import _linearForms
    return _linearForms.SlonczewskiSecond(self._phi, self._j_cf \
      , self.parameters.P, self._p_cf, gf1, gf2) \
      + _linearForms.SlonczewskiFirst(self._phi, self._j_cf, self.parameters.P \
      , self._p_cf, gf1, gf2) \
      + _linearForms.SlonczewskiFirst(self._phi, self._jprime_cf \
      , self.parameters.P, self._p_cf, gf1, gf1)


#------------------------------------------------------------------------------#


################################################################################
