from . import _details
from . import _forms

################################################################################


class _Integrator( \
  _details.interfaces.communicationInterface.CommunicationInterface, \
  _details.interfaces.computeInterface.ComputeInterface, \
  _details.interfaces.contributionInterface.ContributionInterface, \
  _details.interfaces.parameterInterface.ParameterInterface, \
  _details.interfaces.writeInterface.WriteInterface,
  _forms.blf_common.BilinearForms_Common, \
  _forms.lf_common.LinearForms_Common, \
  _forms.blf_magnetization.BilinearForms_Magnetization, \
  _forms.lf_magnetization.LinearForms_Magnetization):
  

#------------------------------------------------------------------------------#


  def __init__(self, geometry, parameters = None, output = None, numthreads = 1):

    # must be called before all scipy, numpy, ngsolve, bempp, ... imports
    self._SetNumThreads(numthreads)
    self._numthreads = numthreads

    from ngsolve.ngstd import Timer
    from ngsolve import ngsglobals
    from .. import _setup
    ngsglobals.msg_level = 0

    self._geometry = geometry
    self.parameters = (_setup.Parameters() if (parameters == None) else parameters)
    print(parameters)
    self.output = (_setup.OutputOptions() if (output == None) else output)
    
    self._nonTrivialMembers = ("_geometry", "parameters", "output")
    
    self._Mag = None
    
    self._C_ex = None
    self._C_ani = None
    self._C_st = None
    self._C_dm = None
    self._k = None
    self._T_end = None
    self._t = None
    self._time_cf = None
    self._hext_cf = None
    self._hext_impl_cf = None
    self._easy_cf = None
    self._p_cf = None
    self._j_cf = None
    self._jprime_cf = None
    
    self._V = None
    self._X = None

    self._psi = None
    self._phi = None
    
    self._Dpsi = None
    self._Dphi = None

    self._a_stat = None
    self._A_stat = None
    self._a_instat = None
    self._A_instat = None

    self._callback = None
    self._fixedPointCallback = None
    self._itCounter = _details.IterationCounter()
    self._fixedPointItCounter = _details.IterationCounter()
    self._preconditioner = None
    self._guess = None
    self._solution = None
    
    self._f_explicit = None
    
    self._LHS = None
    self._RHS = None

    self._newFolder = True
    
    self._compTimeNow = 0.0
    self._compTimeTotal = 0.0
        
    self._counter = 0 
    self._startCounter = 0
    self._steps = 0 

    self._maxwellTimer = Timer("magnetostaticField")

    self._saveMagEvery = None

    self._maxwellCouplingChanged = False

    self._externalFieldRecordingPoint = None


#------------------------------------------------------------------------------#


  def _CheckForBadParameterCombinations(self):
    import sys
    ## TODO search for all the prints and sys.exits --> replace with raise(...)

    from . import tps2
    if self.parameters.dmImplicitTreatment:
      if not(isinstance(self, tps2.TPS2)):
        raise ValueError("dmImplicitTreatment=True only available for TPS2")
      if not(self.parameters.dmCoupling == "bulk"):
        raise ValueError("dmImplicitTreatment=True only available for bulkDMI")


#------------------------------------------------------------------------------#


  _PhysicalToMathematical = _details.PhysicalToMathematical


#------------------------------------------------------------------------------#


  def _ExistsMagnetization(self):
    import sys
    if (self._Mag == None):
      print("Error in _Integrator.ExistsMagnetization(self)")
      print("No magnetization specified. --> Terminate.")
      sys.exit()
  
  
#------------------------------------------------------------------------------#


  def _ResetIterationCounter(self):
    if isinstance(self._itCounter, _details.IterationCounter):
      self._itCounter.Reset()


#------------------------------------------------------------------------------#


  def _ResetFixedPointIterationCounter(self):
    if isinstance(self._fixedPointItCounter, _details.IterationCounter):
      self._fixedPointItCounter.Reset()

  
#------------------------------------------------------------------------------#


  def _SetNumThreads(self, numthreads):
    import os
    os.environ['OMP_NUM_THREADS'] = str(numthreads)
    os.environ['NGS_NUM_THREADS'] = str(numthreads)
    os.environ['BEMPP_NUM_THREADS'] = str(numthreads)

    from mpi4py import MPI # import here once before bempp to avoid Rank error
    from ngsolve import ngsglobals
    ngsglobals.numthreads = numthreads


#------------------------------------------------------------------------------#


  def _SetUpOutput(self):
    pnt = self.parameters.externalFieldRecordingPoint
    if pnt == "first mesh point":
      self._externalFieldRecordingPoint = \
        list(self._geometry.GetMesh().ngmesh.Points()[1])
    else:
      self._externalFieldRecordingPoint = ( \
        pnt[0] / self._geometry.scaling, \
        pnt[1] / self._geometry.scaling, \
        pnt[2] / self._geometry.scaling)

    if(self._saveMagEvery == None): return
    if(self._saveMagEvery == 0): return

    self.output.saveatevery = \
      round(self._saveMagEvery / self.parameters.timeStepSize)


#------------------------------------------------------------------------------#


################################################################################
