### TPS tmeplate ###
from . import _integrator
from . import _details


################################################################################


class TPSX(_integrator._Integrator, \
  _details.interfaces.tpsPreconditioner.TPSPreconditioner):
  

#------------------------------------------------------------------------------#

  
  def GetReady(self):

    _integrator._Integrator.GetReady(self)

    self._UpdateAuxiliaryFunctions()
    self._SetUpPreconditioner()


#------------------------------------------------------------------------------#


  def _StepPrepareWithTaskManager(self):

    self._ComputeMaxwell()
    self._AssembleSystem()


#------------------------------------------------------------------------------#


  def _AssembleLHS(self):
    self._Assemble_A_instat()


#------------------------------------------------------------------------------#


  def _StepPrepareWithoutTaskManager(self):

    self._Compute_Q()
    self._ComputeLHS()
    self._ComputeRHS()
    self._UpdatePreconditioner()


#------------------------------------------------------------------------------#


  def _ComputeLHS(self):
    self._SetLHS_3D()
    self._LHS = self._Q.dot(self._LHS_3D.dot(self._Q.transpose()))


#------------------------------------------------------------------------------#


  def _SetLHS_3D(self):
    self._LHS_3D = self._A_stat + self._A_instat


#------------------------------------------------------------------------------#


  def _ComputeRHS(self):
    self._SetRHS_3D()
    self._RHS = self._Q.dot(self._RHS_3D)


#------------------------------------------------------------------------------#


  def _UpdateGuess(self):
    if(self.parameters.guessType == "old"):
      self._UpdateGuess_Old()
    elif(self.parameters.guessType == None):
      self._UpdateGuess_None()
    else:
      raise ValueError("guessType = ", self.parameters.guessType, "not known")


#------------------------------------------------------------------------------#


  def _UpdateGuess_Old(self):
    self._guess = self._solution


#------------------------------------------------------------------------------#


  def _UpdateGuess_None(self):
    self._guess = None


#------------------------------------------------------------------------------#

  
  def _SetQmatTransformationType(self, trafoType):
    import sys

    if trafoType == "householder":
      self._Q_proj = self._H_proj
    elif trafoType == "rotation":
      self._Q_proj = self._R_proj
    elif trafoType == "staticHouseholder":
      self._Q_proj = self._S_proj
    else:
      print("From _SetQmatTransformationType: parameters.QmatType =", trafoType)
      print("unknown --> Terminate")
      sys.exit()


#------------------------------------------------------------------------------#

  
  def _SetQmatAxis(self, axis):
    import sys

    if axis == "e1" or axis == "e1minus":
      self._Q_axisT = self._Q_axisTe1
    elif axis == "e2" or axis == "e2minus":
      self._Q_axisT = self._Q_axisTe2
    elif axis == "e3" or axis == "e3minus":
      self._Q_axisT = self._Q_axisTe3
    elif axis == "e1plus":
      self._Q_axisT = self._Q_axisTe1plus
    elif axis == "e2plus":
      self._Q_axisT = self._Q_axisTe2plus
    elif axis == "e3plus":
      self._Q_axisT = self._Q_axisTe3plus
    elif axis == "adaptive":
      self._Q_ChooseAxisAdaptive()
    else:
      print("From _SetQmatAxis: parameters.QmatAxis =", axis)
      print("unknown --> Terminate")
      sys.exit()


#------------------------------------------------------------------------------#


  @classmethod
  def _Q_axisTe1(cls, q):
    if len(q) == 3:
      return [q[2], q[1], q[0]]

    return [q[4], q[5], q[2], q[3], q[0], q[1]]


#------------------------------------------------------------------------------#


  @classmethod
  def _Q_axisTe2(cls, q):
    if len(q) == 3:
      return [q[0], q[2], q[1]]

    return [q[0], q[1], q[4], q[5], q[2], q[3]]


#------------------------------------------------------------------------------#


  @classmethod
  def _Q_axisTe3(cls, q):
    return q


#------------------------------------------------------------------------------#


  @classmethod
  def _Q_axisTe1plus(cls, q):
    if len(q) == 3:
      return [-q[2], q[1], -q[0]]

    return [-q[4], -q[5], q[2], q[3], -q[0], -q[1]]


#------------------------------------------------------------------------------#


  @classmethod
  def _Q_axisTe2plus(cls, q):
    if len(q) == 3:
      return [q[0], -q[2], -q[1]]

    return [q[0], q[1], -q[4], -q[5], -q[2], -q[3]]


#------------------------------------------------------------------------------#


  @classmethod
  def _Q_axisTe3plus(cls, q):
    if len(q) == 3:
      return [q[0], q[1], -q[2]]

    return [q[0], q[1], q[2], q[3], -q[4], -q[5]]


#------------------------------------------------------------------------------#


  def _Q_ChooseAxisAdaptive(self):
    import numpy as np
    d = np.zeros(6, dtype=float)

    for j in range(3):
      d[2*j] = 1.0 + (self._Mag.m.components[j].vec.FV().NumPy()).min()
      d[2*j+1] = 1.0 - (self._Mag.m.components[j].vec.FV().NumPy()).max()

    self._adaptiveAxis_d_vals = d.tolist()

    idx = d.argmax()
    self._adaptiveGamma = d[idx]

    if idx == 0:
      self._Q_axisT = self._Q_axisTe1
      self._adaptiveAxis = "-1"
    elif idx == 1:
      self._Q_axisT = self._Q_axisTe1plus
      self._adaptiveAxis = "+1"
    elif idx == 2:
      self._Q_axisT = self._Q_axisTe2
      self._adaptiveAxis = "-2"
    elif idx == 3:
      self._Q_axisT = self._Q_axisTe2plus
      self._adaptiveAxis = "+2"
    elif idx == 4:
      self._Q_axisT = self._Q_axisTe3
      self._adaptiveAxis = "-3"
    elif idx == 5:
      self._Q_axisT = self._Q_axisTe3plus
      self._adaptiveAxis = "+3"


#------------------------------------------------------------------------------#

  
  def _SetUpAuxiliaryFunctions(self):
    from commics._gridFunctions import Derivative

    self._Mag.v = Derivative(self._GetMesh(), self._V, self._X \
      , self.parameters.maxwellCoupling, self._solvetol)

    _details.interfaces.tpsPreconditioner.TPSPreconditioner._SetUpAuxiliaryFunctions(self)

    self._Q = None
    self._Q_proj = None
    self._Q_axisT = None


#------------------------------------------------------------------------------#


  def _UpdateAuxiliaryFunctions(self):
    self._SetQmatTransformationType(self.parameters.QmatType)
    self._SetQmatAxis(self.parameters.QmatAxis)


#------------------------------------------------------------------------------#


  def _UpdateMaxwellCoupling(self):

    _integrator._Integrator._UpdateMaxwellCoupling(self)

    self._Mag.v.UpdateMaxwellCoupling(self.parameters.maxwellCoupling \
      , self._solvetol)


#------------------------------------------------------------------------------#


  def _StepSolve(self):
    self._Compute_v()


#------------------------------------------------------------------------------#


  def _Assemble_A_instat(self):
    self._a_instat.Assemble()
    self._A_instat.data = self._a_instat.mat.COO()[2].NumPy()


#------------------------------------------------------------------------------#


  def _AssembleRHS(self):
    import sys
    print("From _tpsx.AssembleRHS():")
    print("I'm only the base class, I don't know what to do.")
    print("--> Terminate")
    sys.exit()


#------------------------------------------------------------------------------#


  def _ExchangeTerm(self):
    return -self._C_ex * self._Exchange(self._Mag.Dm)


#------------------------------------------------------------------------------#


  def _ExplicitTerms(self):
    return self._AppliedTerm() + self._OerstedFieldTerm()


#------------------------------------------------------------------------------#


  def _AppliedTerm(self):
    return self._Applied()


#------------------------------------------------------------------------------#


  def _OerstedFieldTerm(self):
    return self._Oersted(self._hc)


#------------------------------------------------------------------------------#


  def _H_proj(self, x, y, z):
    x, y, z = self._Q_axisT([x, y, z])

    sigma = 1 if z > 0 else -1
        
    scale = 1.0 + abs(z)

    R0 = 1.0 - x*x / scale
    R12 = - x*y / scale 
    R3 = 1.0 - y*y / scale
    R4 = - x*(z + sigma) / scale
    R5 = - y*(z + sigma) / scale
    
    return self._Q_axisT([R0, R12, R12, R3, R4, R5])
  
  
#------------------------------------------------------------------------------#


  def _R_proj(self, x, y, z):
    epsilon = 1.0e-12

    x, y, z = self._Q_axisT([x, y, z])
        
    scale = 1.0+z

    R0 = 1.0 
    R12 = 0.0 
    R3 = 1.0 
    R4 = -x
    R5 = -y
    
    if ( abs(scale) > epsilon ):
      R0 -= x*x/scale
      R12 -= x*y/scale
      R3 -= y*y/scale

    return self._Q_axisT([R0, R12, R12, R3, R4, R5])
  
  
#------------------------------------------------------------------------------#


  def _S_proj(self, x, y, z):
    epsilon = 1.0e-12
    x, y, z = self._Q_axisT([x, y, z])

    scale = 1.0+z
        
    R0 = 1.0
    R12 = 0.0
    R3 = 1.0
    R4 = 0.0
    R5 = 0.0
    
    if ( abs(scale) > epsilon ):
      R0 -= x*x / scale
      R12 -=  x*y / scale
      R3 -= y*y / scale
      R4 -= x
      R5 -= y

    return self._Q_axisT([R0, R12, R12, R3, R4, R5])
  
  
#------------------------------------------------------------------------------#


  def _Compute_Q(self):
    import scipy.sparse as sp
    import numpy as np
    N = self._Mag.m.components[0].space.ndof
        
    m_fv = self._Mag.m.vec.FV().NumPy()
    Q_data = np.empty(6*N, dtype=float)
    D = np.empty(6*N, dtype=float)

    if self.parameters.QmatAxis == "adaptive":
      self._Q_ChooseAxisAdaptive()

    for j in range(N):
      D[6*j:6*(j+1)] \
        = self._Q_proj(m_fv[j], m_fv[j+N], m_fv[j+2*N])
   
    E = D.reshape((6,N), order='F')
    Q_data[:2*N] = E[:2,:].reshape((2*N,), order='F')
    Q_data[2*N:4*N] = E[2:4,:].reshape((2*N,), order='F')
    Q_data[4*N:] = E[4:,:].reshape((2*N,), order='F')

    tmp = np.arange(2*N)
    Q_rows = np.append(tmp, [tmp, tmp])
    Q_cols = np.repeat(np.arange(3*N), 2)
        
    self._Q = sp.csr_matrix( (Q_data, (Q_rows, Q_cols)), shape=(2*N,3*N) )   
  

#------------------------------------------------------------------------------#


################################################################################
