###  KW1+AB ###
from . import kw1
from . import _forms


################################################################################


class KW1AB(kw1.KW1, _forms.lf_old.LinearForms_Old):
  # description of KW1AB


#------------------------------------------------------------------------------#

  
  def __init__(self, geometry, parameters = None, output = None, numthreads = 1):
    
    kw1.KW1.__init__(self, geometry, parameters, output, numthreads)

    self._nowImplicit = True


#------------------------------------------------------------------------------#


  def GetReady(self):
    self._tsm_new = 1.0 + self._theta
    self._tsm_old = -self._theta
    
    kw1.KW1.GetReady(self)
    self._nowImplicit = True


#------------------------------------------------------------------------------#


  def _SetUpAuxiliaryFunctions(self):
    super()._SetUpAuxiliaryFunctions()

    self._SetUpOldMagnetization()


#------------------------------------------------------------------------------#

  
  def _SetUpOldMagnetization(self):
    from commics._gridFunctions import Magnetization
    self._Mag_old = Magnetization(self._geometry, self._V, self._X \
      , self._Mag, self.parameters.maxwellCoupling, self._solvetol)


#------------------------------------------------------------------------------#


  def _UpdateMaxwellCoupling(self):

    kw1.KW1._UpdateMaxwellCoupling(self)

    self._Mag_old.UpdateMaxwellCoupling(self.parameters.maxwellCoupling \
      , self._solvetol)


#------------------------------------------------------------------------------#


  def _StepFollowUp(self):
    
    kw1.KW1._StepFollowUp(self)

    self._nowImplicit = False


#------------------------------------------------------------------------------#


  def _StepUpdate(self):
    self._Mag_old.SetFromOther(self._Mag)

    super()._StepUpdate()


#------------------------------------------------------------------------------#


  def _UpdateGuess(self):
    if(self.parameters.guessType == "old"):
      self._UpdateGuess_Old()
    elif(self.parameters.guessType == None):
      self._UpdateGuess_None()
    else:
      raise ValueError("guessType = ", self.parameters.guessType, "not known")


#------------------------------------------------------------------------------#


  def _UpdateGuess_Old(self):
    self._guess = self._Mag.v.gf.vec.FV().NumPy()


#------------------------------------------------------------------------------#


  def _UpdateGuess_None(self):
    self._guess = None


#------------------------------------------------------------------------------#


  def _SetUpSystems(self):
    from ngsolve import SymbolicLFI, LinearForm

    super()._SetUpSystems()

    #lf
    rhs_ab_cf = self._KW_ExchangeTerm()
    rhs_ab_cf -= self._ab_LinearTerms_pi()
    rhs_ab_cf -= self._ab_NonLinearTerms_Pi()
    rhs_ab_cf -= self._KW_ExplicitTerms()

    self._ab_f_parts = LinearForm(self._X)
    self._ab_f_parts += SymbolicLFI(rhs_ab_cf.Compile(realcompile=True, wait=True))



#------------------------------------------------------------------------------#


  def _StepPrepareWithoutTaskManager(self):
    self._ComputeSystemFromParts()


#------------------------------------------------------------------------------#


  def _ComputeSystemFromParts(self):
    self._Compute_LHS_instat()
    self._LHS = self._LHS_stat + self._LHS_instat

    self._Compute_RHS()


#------------------------------------------------------------------------------#

  
  def _Compute_RHS(self):
    self._RHS = self._Sh_MhInv(self._ab_f_parts.vec.FV().NumPy())


#------------------------------------------------------------------------------#


  def _Compute_v(self):
    import scipy.sparse.linalg
    if(self._nowImplicit):
      kw1.KW1._Compute_v(self)
      return

    self._Mag.v.gf.vec.FV().NumPy()[:], succ = scipy.sparse.linalg.gmres( \
      self._LHS, self._RHS \
      , x0=self._guess, tol=self._solvetol, maxiter=4000 \
      , M=self._preconditioner, callback=self._callback)

    self._UpdateGuess()
    

#------------------------------------------------------------------------------#


  def _AssemblePartsRHS(self):
    self._Assemble_f_ab()


#------------------------------------------------------------------------------#


  def _AssembleFixPartsRHS(self):
    if self._nowImplicit:
      kw1.KW1._AssembleFixPartsRHS(self)
    else:
      self._AssemblePartsRHS()


#------------------------------------------------------------------------------#


  def _Assemble_f_ab(self):
    self._ab_f_parts.Assemble()


#------------------------------------------------------------------------------#


  def _ab_LinearTerms_pi(self):
    return self._tsm_new * self._LinearTerms_pi() \
      + self._tsm_old * self._OldLinearTerms_pi()


#------------------------------------------------------------------------------#


  def _ab_NonLinearTerms_Pi(self):
    return self._NonLinearTerms_Pi() \
      + (self._tsm_new - 1.0) * self._NowDerivNonLinearTerms() \
      + self._tsm_old * self._OldDerivNonLinearTerms()


#------------------------------------------------------------------------------#


################################################################################
