### KW2 ###
from . import kw1

################################################################################


class KW2(kw1.KW1):


#------------------------------------------------------------------------------#


  def _SetUpAuxiliaryFunctions(self):
    from ngsolve import GridFunction

    super()._SetUpAuxiliaryFunctions()

    self._SetUpEta()

    self._heff_m_kv_gf = GridFunction(self._X)

    self._eta_guess = None
    self._eta_preconditioner = None


#------------------------------------------------------------------------------#


  def _SetUpEta(self):
    from commics._gridFunctions import Magnetization

    self._Mag_eta = Magnetization(self._geometry, self._V, self._X \
      , self._Mag, self.parameters.maxwellCoupling, self._solvetol)


#------------------------------------------------------------------------------#


  def _SetUpSystems(self):
    from commics._tools import ScipySparseFromNgMatrix
    from ngsolve import IntegrationRule, BilinearForm, SymbolicBFI \
      , LinearForm, SymbolicLFI, TaskManager

    super()._SetUpSystems()

    massLumping = IntegrationRule( \
      points = [(0,0,0), (1,0,0), (0,1,0), (0,0,1)] \
      , weights = [1/24, 1/24, 1/24, 1/24] )

    #auxiliary
    aux_heff_m_parts = -self._KW_ExchangeTerm()
    aux_heff_m_parts += self._LinearTerms_pi()
    aux_heff_m_parts += self._NonLinearTerms_Pi()
    aux_heff_m_parts += self._ExplicitTermsMidpoint()

    aux_heff_v_parts = -self._DerivExchangeTermMidpoint()
    aux_heff_v_parts += self._DerivLinearTerms_piMidpoint()
    aux_heff_v_parts += self._DerivNonLinearTerms_PiMidpoint()

    aux_heff_parts = aux_heff_m_parts + aux_heff_v_parts

    self._aux_f_parts = LinearForm(self._X)
    self._aux_f_parts += SymbolicLFI(aux_heff_parts.Compile( \
      realcompile=True, wait=True))


    #blf
    self._update_LHS_stat = self._Get_Update_LhsMass_Matrix()

    update_lhs_instat_cf = self._Update_Lhs_Skew_heff_m_kv_Term()
    update_lhs_instat_cf += self._Update_Lhs_Skew_m_kv_cross_heff_m_kv_Term()

    self._update_a = BilinearForm(self._X)
    self._update_a += SymbolicBFI(update_lhs_instat_cf.Compile( \
      realcompile=True, wait=True), intrule=massLumping)

    with TaskManager():
      self._update_a.Assemble()
    self._update_LHS_instat = ScipySparseFromNgMatrix(self._update_a)
    

    #lf
    update_rhs_cf = self._Update_RhsMassTerm()

    self._update_f = LinearForm(self._X)
    self._update_f += SymbolicLFI(update_rhs_cf.Compile( \
      realcompile=True, wait=True), intrule=massLumping)


#------------------------------------------------------------------------------#


  def _StepUpdate(self):
    import numpy as np

    self._ComputeEta()

    m_old_gf = np.array(self._Mag.m.vec.FV())
    self._Mag.m.vec.FV().NumPy()[:] = \
      2.0 * self._Mag_eta.m.vec.FV().NumPy() - m_old_gf


#------------------------------------------------------------------------------#

  
  def _ComputeEta(self):
    self._ComputeUpdateSystem()
    self._SolveUpdateSystem()


#------------------------------------------------------------------------------#


  def _ComputeUpdateSystem(self):
    from ngsolve import TaskManager

    with TaskManager():
      self._Mag.v.ComputeMaxwell(self._solvetol)

      self._Compute_Update_LHS()
      self._Compute_Update_RHS()


#------------------------------------------------------------------------------#


  def _Compute_Update_LHS(self):
    self._AssembleUpdatePartsLHS()
    self._heff_m_kv_gf.vec.FV().NumPy()[:] \
      = self._MhInv_diag * self._aux_f_parts.vec.FV().NumPy()
  
    self._AssembleUpdateLHS()

    self._update_LHS = self._update_LHS_stat + self._update_LHS_instat


#------------------------------------------------------------------------------#


  def _AssembleUpdatePartsLHS(self):
    self._aux_f_parts.Assemble()


#------------------------------------------------------------------------------#


  def _AssembleUpdateLHS(self):
    self._update_a.Assemble()
    self._update_LHS_instat.data = self._update_a.mat.COO()[2].NumPy()


#------------------------------------------------------------------------------#


  def _Compute_Update_RHS(self):
    self._AssembleUpdateRHS()
    
    self._update_RHS = self._update_f.vec.FV().NumPy()


#------------------------------------------------------------------------------#


  def _AssembleUpdateRHS(self):
    self._update_f.Assemble()


#------------------------------------------------------------------------------#


  def _SolveUpdateSystem(self):
    import scipy.sparse.linalg

    self._Mag_eta.m.vec.FV().NumPy()[:], succ = scipy.sparse.linalg.gmres( \
      self._update_LHS, self._update_RHS \
      , x0=self._eta_guess, tol=self._solvetol, maxiter=4000 \
      , M=self._eta_preconditioner, callback=self._callback)

    self._UpdateEtaGuess()
    

#------------------------------------------------------------------------------#


  def _UpdateEtaGuess(self): pass


#------------------------------------------------------------------------------#


  def _ExplicitTermsMidpoint(self):
    return self._Applied_mp() + self._KW_OerstedFieldTerm_impl()


#------------------------------------------------------------------------------#


  def _AppliedTermMidpoint(self):
    return self._Applied_mp()


#------------------------------------------------------------------------------#
  

  def _Get_Update_LhsMass_Matrix(self):
    return (2.0 * (1.0+self.parameters.alpha**2) / self._k) * self._Mh


#------------------------------------------------------------------------------#


  def _Update_RhsMassTerm(self):
    return (2.0 * (1.0+self.parameters.alpha**2) / self._k) * self._RhsMass()


#------------------------------------------------------------------------------#


  def _Update_Lhs_Skew_heff_m_kv_Term(self):
    return -self._Skew_heff_m_kv()
    

#------------------------------------------------------------------------------#


  def _Skew_heff_m_kv(self):
    return self._Skew(self._heff_m_kv_gf)


#------------------------------------------------------------------------------#


  def _Update_Lhs_Skew_m_kv_cross_heff_m_kv_Term(self):
    return -self.parameters.alpha * self._Skew_m_kv_cross_heff_m_kv()
    

#------------------------------------------------------------------------------#


  def _Skew_m_kv_cross_heff_m_kv(self):
    from commics._tools.algebra import CrossProd

    return self._Skew(CrossProd(self._Get_m_kv(), self._heff_m_kv_gf))


#------------------------------------------------------------------------------#

  
  def _Get_m_kv(self):
    return self._Mag.m + 0.5 * self._k * self._Mag.v.gf


#------------------------------------------------------------------------------#


################################################################################
