###  MPS+AB ###
from . import mps
from . import _forms


################################################################################


class MPSAB(mps.MPS, _forms.lf_derivative.LinearForms_Derivative, \
  _forms.lf_old.LinearForms_Old):
  '''
  implements IMEX approach from
  Praetorius, D., Ruggeri, M., and Stiftner, B. Convergence of an implicit-explicit midpoint scheme for computational micromagnetics. Accepted for publication in Comput. Math. Appl., preprint available at arXiv:1611.02465, 2018.
  '''


#------------------------------------------------------------------------------#

  
  def __init__(self, geometry, parameters = None, output = None, numthreads = 1):
    
    mps.MPS.__init__(self, geometry, parameters, output, numthreads)

    self._nowImplicit = True


#------------------------------------------------------------------------------#


  def GetReady(self):
    self._tsm_new = 1.0 + self._theta
    self._tsm_old = -self._theta
    
    mps.MPS.GetReady(self)
    self._nowImplicit = True


#------------------------------------------------------------------------------#


  def _SetUpAuxiliaryFunctions(self):
    mps.MPS._SetUpAuxiliaryFunctions(self)

    self._SetUpOldMagnetization()


#------------------------------------------------------------------------------#

  
  def _SetUpOldMagnetization(self):
    from commics._gridFunctions import Magnetization

    self._Mag_old = Magnetization(self._geometry, self._V, self._X \
      , self._Mag, self.parameters.maxwellCoupling, self._solvetol)


#------------------------------------------------------------------------------#


  def _UpdateMaxwellCoupling(self):

    mps.MPS._UpdateMaxwellCoupling(self)

    self._Mag_old.UpdateMaxwellCoupling(self.parameters.maxwellCoupling \
      , self._solvetol)


#------------------------------------------------------------------------------#


  def _UpdateMagnetostatic_insideFixedPointIteration(self):
    pass
  
    
#------------------------------------------------------------------------------#


  def _StepFollowUp(self):
    
    mps.MPS._StepFollowUp(self)

    if self._nowImplicit:
      self._SetToAB()


#------------------------------------------------------------------------------#


  def _StepUpdate(self):
    self._Mag_old.SetFromOther(self._Mag)

    mps.MPS._StepUpdate(self)


#------------------------------------------------------------------------------#


  def _SetUpSystems(self):
    from ngsolve import SymbolicLFI, LinearForm

    mps.MPS._SetUpSystems(self)

    #blf
    lhs_fix_ab_parts_cf = self._LhsFixAbPartsTerm()

    self._lhs_fix_ab_parts = LinearForm(self._X)
    self._lhs_fix_ab_parts += SymbolicLFI(lhs_fix_ab_parts_cf.Compile( \
      realcompile=True, wait=True))


#------------------------------------------------------------------------------#


  def _LhsFixAbPartsTerm(self):
    return self._ab_LinearTerms_pi() +self._ab_NonLinearTerms_Pi() \
      + self._ExplicitTerms()


#------------------------------------------------------------------------------#


  def _ComputeLhsIterAbPartsGf(self):
    self._lhs_iter_parts_gf.vec.FV().NumPy()[:] = -self._C_ex \
      * (self._DiscreteLaplacianMatrix.dot(self._Mag_eta.m.vec.FV().NumPy()))


#------------------------------------------------------------------------------#


  def _ab_LinearTerms_pi(self):
    return self._tsm_new * self._LinearTerms_pi() \
      + self._tsm_old * self._OldLinearTerms_pi()


#------------------------------------------------------------------------------#


  def _ab_NonLinearTerms_Pi(self):
    return self._NonLinearTerms_Pi() \
      + (self._tsm_new - 1.0) * self._NowDerivNonLinearTerms() \
      + self._tsm_old * self._OldDerivNonLinearTerms()


#------------------------------------------------------------------------------#


  def _SetToAB(self):
    self._nowImplicit = False
    self._lhs_fix_parts = self._lhs_fix_ab_parts


#------------------------------------------------------------------------------#


  def _ComputeLhsIterPartsGf(self):
    if self._nowImplicit:
      mps.MPS._ComputeLhsIterPartsGf(self)
    else:
      self._ComputeLhsIterAbPartsGf()


#------------------------------------------------------------------------------#


  def _ComputeLhsIterAbPartsGf(self):
    self._lhs_iter_parts_gf.vec.FV().NumPy()[:] = -self._C_ex \
      * (self._DiscreteLaplacianMatrix.dot(self._Mag_eta.m.vec.FV().NumPy()))


#------------------------------------------------------------------------------#


################################################################################
