#!/bin/sh
################################################################################
SOFTWARE_DIR=~/Software
NR_MAKE_THREADS=2
PYTHON_INSTALLATION=/usr/bin/python3
WRITE_PATHS_TO=~/.bashrc
################################################################################
NGSOLVE_GIT_VERSION_HASH="0419c0285dae6589c32a5ddfaf0dea2e89a5e3b7"
BEMPP_GIT_VERSION_HASH="ce2ffc1ce52f07bf5890b19fb7c4389a7bb1031a"
NGBEM_GIT_VERSION_HASH="499138ca5dd17e812d3bdfdc0556afbff4ca340c"
COMMICS_GIT_VERSION_HASH="cb67b268c0cb56e1b532087284270148217f94fd"
################################################################################

# install requirements
sudo apt update
sudo apt upgrade
sudo apt install python3-tk libpython3-dev libxmu-dev tk-dev tcl-dev cmake git g++-8 gcc-8 libglu1-mesa-dev liblapacke-dev mpich libboost-all-dev libtbb-dev python3-pip cython3 patchelf
export CXX=$(which g++-8)
export CC=$(which gcc-8)
sudo pip3 install numpy scipy mpi4py pyamg cython

#ngsolve installation
mkdir -p ${SOFTWARE_DIR}/ngsolve
cd ${SOFTWARE_DIR}/ngsolve
git clone https://github.com/NGSolve/ngsolve.git ngsolve-src
mkdir ngsolve-install
mkdir ngsolve-build
cd ngsolve-src
git checkout ${NGSOLVE_GIT_VERSION_HASH}
git submodule update --init --recursive
cd ../ngsolve-build
cmake -DCMAKE_INSTALL_PREFIX=../ngsolve-install ../ngsolve-src
make -j${NR_MAKE_THREADS}
make install
echo "export NETGENDIR=${SOFTWARE_DIR}/ngsolve/ngsolve-install/bin" >> ${WRITE_PATHS_TO} 
echo "export PYTHONPATH=\${PYTHONPATH}:${SOFTWARE_DIR}/ngsolve/ngsolve-install/lib/python3/dist-packages" >> ${WRITE_PATHS_TO} 
echo "export PATH=\${PATH}:\${NETGENDIR}" >> ${WRITE_PATHS_TO} 

#bem++ installation
mkdir -p ${SOFTWARE_DIR}/eigen
cd ${SOFTWARE_DIR}/eigen
wget https://gitlab.com/libeigen/eigen/-/archive/3.3.9/eigen-3.3.9.tar.bz2
tar -xjf eigen-3.3.9.tar.bz2
mv eigen-3.3.9 eigen-headers
mkdir -p ${SOFTWARE_DIR}/bempp
cd ${SOFTWARE_DIR}/bempp
git clone https://bitbucket.org/bemppsolutions/bempp.git bempp-src
cd bempp-src
git checkout ${BEMPP_GIT_VERSION_HASH}
cd ..
mkdir bempp-install
mkdir bempp-build
cd bempp-build
cmake ../bempp-src -DEIGEN3_INCLUDE_DIR=${SOFTWARE_DIR}/eigen/eigen-headers -DPYTHON_EXECUTABLE=${PYTHON_INSTALLATION} -DCMAKE_INSTALL_PREFIX=${SOFTWARE_DIR}/bempp/bempp-install
make -j${NR_MAKE_THREADS}
make install
cd ${SOFTWARE_DIR}
PYTHON3_SUB_BEMPP_FOLDER=$(ls ${SOFTWARE_DIR}/bempp/bempp-install/lib | grep python)
echo "export PYTHONPATH=\${PYTHONPATH}:${SOFTWARE_DIR}/bempp/bempp-install/lib/${PYTHON3_SUB_BEMPP_FOLDER}/site-packages" >> ${WRITE_PATHS_TO}

#ngbem clone
cd ${SOFTWARE_DIR}
git clone https://github.com/arieder/ngbem.git ngbem
cd ngbem
git checkout ${NGBEM_GIT_VERSION_HASH}
echo "export PYTHONPATH=\${PYTHONPATH}:${SOFTWARE_DIR}/ngbem" >> ${WRITE_PATHS_TO} 

#commics clone
mkdir -p ${SOFTWARE_DIR}/commics
cd ${SOFTWARE_DIR}/commics
git clone https://geras.asc.tuwien.ac.at/cpfeiler/commics.git
cd commics
git checkout ${COMMICS_GIT_VERSION_HASH}
echo "export PYTHONPATH=\${PYTHONPATH}:${SOFTWARE_DIR}/commics" >> ${WRITE_PATHS_TO} 
